@@ -1,6 +1,6 @@
 use serde::{Deserialize, Serialize};
 
-use std::fmt;
+use std::{fmt, ops};
 
 use crate::sources::*;
 use crate::terms::*;
@@ -33,7 +33,7 @@ pub enum ErrorKind {
     Runtime(RuntimeError),
     Operational(OperationalError),
     Parameter(ParameterError),
-    RolesValidation(RolesValidationError),
+    Validation(ValidationError),
 }
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
@@ -56,7 +56,8 @@ impl PolarError {
                 | ParseError::WrongValueType { loc, .. }
                 | ParseError::ReservedWord { loc, .. }
                 | ParseError::DuplicateKey { loc, .. }
-                | ParseError::SingletonVariable { loc, .. } => {
+                | ParseError::SingletonVariable { loc, .. }
+                | ParseError::ResourceBlock { loc, .. } => {
                     let (row, column) = crate::lexer::loc_to_pos(&source.src, *loc);
                     self.context.replace(ErrorContext {
                         source: source.clone(),
@@ -76,8 +77,39 @@ impl PolarError {
             }
             _ => {}
         }
+
+        // Augment ResourceBlock errors with relevant snippets of parsed Polar policy.
+        if let ErrorKind::Parse(ParseError::ResourceBlock {
+            ref mut msg,
+            ref ranges,
+            ..
+        }) = self.kind
+        {
+            if let Some(source) = source {
+                match ranges.len() {
+                    // If one range is provided, print it with no label.
+                    1 => {
+                        let first = &source.src[ranges[0].clone()];
+                        msg.push_str(&format!("\t{}\n", first));
+                    }
+                    // If two ranges are provided, label them `First` and `Second`.
+                    2 => {
+                        let first = &source.src[ranges[0].clone()];
+                        msg.push_str(&format!("\tFirst:\n\t\t{}\n", first));
+                        let second = &source.src[ranges[1].clone()];
+                        msg.push_str(&format!("\tSecond:\n\t\t{}\n", second));
+                    }
+                    _ => (),
+                }
+            }
+        }
+
         self
     }
+
+    pub fn unimplemented(msg: String) -> Self {
+        OperationalError::Unimplemented { msg }.into()
+    }
 }
 
 impl From<ParseError> for PolarError {
@@ -116,10 +148,10 @@ impl From<ParameterError> for PolarError {
     }
 }
 
-impl From<RolesValidationError> for PolarError {
-    fn from(err: RolesValidationError) -> Self {
+impl From<ValidationError> for PolarError {
+    fn from(err: ValidationError) -> Self {
         Self {
-            kind: ErrorKind::RolesValidation(err),
+            kind: ErrorKind::Validation(err),
             context: None,
         }
     }
@@ -127,6 +159,12 @@ impl From<RolesValidationError> for PolarError {
 
 pub type PolarResult<T> = std::result::Result<T, PolarError>;
 
+impl<T> From<PolarError> for PolarResult<T> {
+    fn from(err: PolarError) -> Self {
+        Err(err)
+    }
+}
+
 impl std::error::Error for PolarError {}
 
 impl fmt::Display for PolarError {
@@ -136,7 +174,7 @@ impl fmt::Display for PolarError {
             ErrorKind::Runtime(e) => write!(f, "{}", e)?,
             ErrorKind::Operational(e) => write!(f, "{}", e)?,
             ErrorKind::Parameter(e) => write!(f, "{}", e)?,
-            ErrorKind::RolesValidation(e) => write!(f, "{}", e)?,
+            ErrorKind::Validation(e) => write!(f, "{}", e)?,
         }
         if let Some(ref context) = self.context {
             write!(f, "{}", context)?;
@@ -192,6 +230,13 @@ pub enum ParseError {
         loc: usize,
         name: String,
     },
+    ResourceBlock {
+        loc: usize,
+        msg: String,
+        /// Set of source ranges to augment the error message with relevant snippets of the parsed
+        /// Polar policy.
+        ranges: Vec<ops::Range<usize>>,
+    },
 }
 
 impl fmt::Display for ErrorContext {
@@ -254,6 +299,9 @@ impl fmt::Display for ParseError {
                     name, name
                 )
             }
+            Self::ResourceBlock { msg, .. } => {
+                write!(f, "{}", msg)
+            }
         }
     }
 }
@@ -337,18 +385,22 @@ impl fmt::Display for RuntimeError {
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
 pub enum OperationalError {
-    Unimplemented(String),
+    Unimplemented {
+        msg: String,
+    },
     Unknown,
 
     /// An invariant has been broken internally.
-    InvalidState(String),
+    InvalidState {
+        msg: String,
+    },
 }
 
 impl fmt::Display for OperationalError {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
-            Self::Unimplemented(s) => write!(f, "{} is not yet implemented", s),
-            Self::InvalidState(s) => write!(f, "Invalid state: {}", s),
+            Self::Unimplemented { msg } => write!(f, "{} is not yet implemented", msg),
+            Self::InvalidState { msg } => write!(f, "Invalid state: {}", msg),
             Self::Unknown => write!(
                 f,
                 "We hit an unexpected error.\n\
@@ -369,10 +421,21 @@ impl fmt::Display for ParameterError {
 }
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
-pub struct RolesValidationError(pub String);
+pub enum ValidationError {
+    InvalidRule { rule: String, msg: String },
+    InvalidRuleType { rule_type: String, msg: String },
+    // TODO(lm|gj): add ResourceBlock and SingletonVariable.
+}
 
-impl fmt::Display for RolesValidationError {
+impl fmt::Display for ValidationError {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "Oso Roles Validation Error: {}", self.0)
+        match self {
+            Self::InvalidRule { rule, msg } => {
+                write!(f, "Invalid rule: {} {}", rule, msg)
+            }
+            Self::InvalidRuleType { rule_type, msg } => {
+                write!(f, "Invalid rule type: {} {}", rule_type, msg)
+            }
+        }
     }
 }
