@@ -9,6 +9,7 @@ pub mod macros;
 
 mod bindings;
 mod counter;
+pub mod data_filtering;
 mod debugger;
 pub mod error;
 pub mod events;
@@ -22,11 +23,11 @@ mod numerics;
 pub mod parser;
 mod partial;
 pub mod polar;
+mod resource_block;
 mod rewrites;
-pub mod roles_validation;
 pub mod rules;
 mod runnable;
-mod sources;
+pub mod sources;
 pub mod terms;
 pub mod traces;
 mod visitor;
